package tp.impl;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import tp.InvalidTransactionException;
import tp.RM;
import tp.TM;
import tp.Transaction;
import tp.TransactionAbortedException;

/**
 * <h3>Your Transaction Manager</h3>
 *
 * A rudimentary implementation is provided for your convenience.
 */
public class MyTM extends CSEP545ServiceObject implements TM {
    private static final long serialVersionUID = -3864279262065640856L;

    /**
     * keep track of resource managers
     */
    private transient Set<RM> resourceManagers = Collections.synchronizedSet(new HashSet<RM>());

    public MyTM() {
    }
    
    @Override
    public Transaction start() throws RemoteException {
        return new Transaction();
    }

    @Override
    public void commit(Transaction xid) throws InvalidTransactionException,
            TransactionAbortedException, RemoteException {
        System.out.println(xid+" : COMMIT");
    }

    @Override
    public void abort(Transaction xid) throws RemoteException {
        System.out.println(xid+" : ABORT");
    }

    @Override
    public boolean enlist(Transaction xid, String enlistingRM)
            throws TransactionAbortedException, InvalidTransactionException,
            RemoteException {
        return false;
    }

    // RELATED TO SERVICE
    
    /**
     * look up RMI registry to cache remote reference of
     * RM represented by <code>rm</code>. This makes easier to
     * coordinate two-phase commit.
     * @param rm the name of RM to register
     */
    @Override
    public void register(String rm) throws RemoteException {
        System.out.println("Register "+rm);
        try {
            ServiceID remote = ServiceID.parse(rm);
            if ( remote.getType() != Type.RM )
                throw new IllegalArgumentException(rm+" is not a resource manager!");
            
            RM newRM = lookupRemote(rm);
            resourceManagers.add(newRM);
        } catch (NotBoundException e) {
            e.printStackTrace();
        }
    }
    
    @Override
    public void shutdown() throws RemoteException {
        // TODO DO PROPER SHUTDOWN HERE
    }

    @Override
    protected void init(String[] args) throws Exception {
    }

    @Override
    protected void initStorage() throws Exception {
        // TODO create commit log
    }

    @Override
    protected void recovery() throws Exception {
        // TODO Abort/commit/garbage collect
    }

    @Override
    protected void startUp() throws Exception {
        // TODO start garbage collector?
    }

    @Override
    protected void readyToServe() throws Exception {
    }

    @Override
    public void refresh() throws RemoteException, NotBoundException {
        // do not need to do anything since all RM will
        // call register again on recovery
    }
}
